<?php
/*
Plugin Name: Дримкас
Description: Позволяет фискализировать заказы магазина через обычную кассу от Дримкас (Дримкас-Ф).
Plugin URI: http://wordpress.org/plugins/dreamkas/
Author: Alt-Team
Version: 1.0.0
Author URI: http://alt-team.ru/
*/

//use WC_Payment_Gateways;

final class Dreamkas {

    public $version = '1.0.0';

    const DEFAULT_QUEUE_NAME = 'default';
    const DISCOUNT_NOT_AVAILABLE = 0;

    private static $_instance = null;

    public static function instance() {
        if (is_null(self::$_instance) ) {
            self::$_instance = new self();
        }
        return self::$_instance;
    }

    public function __construct()
    {
        $this->define('DREAMKAS_ABSPATH', plugin_dir_path( __FILE__));
        $this->define('DREAMKAS_ABSPATH_VIEWS', plugin_dir_path( __FILE__) . 'includes/views/');
        $this->define('DREAMKAS_BASENAME', plugin_basename( __FILE__ ));

        $this->includes();
        $this->hooks();
        $this->wp_hooks();
        $this->wp_endpoints();
        $this->load_options();
        $this->init();
    }

    public function wp_hooks()
    {
        register_activation_hook( __FILE__, array('Dreamkas_Install', 'activation'));
        add_action('woocommerce_order_status_' . get_option('dreamkas_fiscalize_on_order_status'), array($this, 'fiscalize'));
    }

    public function wp_endpoints()
    {
        //add_filter('query_vars', array($this, 'add_query_vars'), 0);
        add_action('init', array($this, 'add_endpoint'), 0);
        add_action('parse_request', array($this, 'handle_requests'), 0);
    }

    public function hooks()
    {
        add_action('dreamkas_action_success', array($this, 'action_success'));
        add_action('dreamkas_action_fail', array($this, 'action_fail'));
        //add_action('dreamkas_report_create', array($this, 'report_create'), 10, 4);
        //add_action('dreamkas_report_update', array($this, 'report_update'), 10, 3);
    }

    public function includes()
    {
        require_once(DREAMKAS_ABSPATH . 'includes/class-dreamkas-install.php');
        //require_once('debug.php');
        
        if (is_admin()) {
            require_once(DREAMKAS_ABSPATH . 'includes/class-dreamkas-admin.php');
            add_action('init', array( 'Dreamkas_Admin', 'init'));
        }
    }

    private function define($name, $value)
    {
        if (!defined( $name )) {
            define( $name, $value );
        }
    }

    public function load_options() {
        $this->access_token = get_option('dreamkas_access_token');
    }

    public function init()
    {
        do_action('before_dreamkas_init');
        do_action('dreamkas_init');
    }

    public function taxSystems() {
		return array(
			'DEFAULT' => 'Общая',
			'SIMPLE' => 'Упрощенная доход',
			'SIMPLE_WO' => 'Упрощенная доход минус расход',
			'ENVD' => 'Единый налог на вмененный доход',
			'AGRICULT' => 'Единый сельскохозяйственный налог',
			'PATENT' => 'Патентная система налогообложения'
		);
	}
        
    public function taxTypes() {
		return array(
			'0' => 'Выберите НДС',
			'NDS_NO_TAX' => 'Без НДС',
			'NDS_0' => 'НДС 0',
			'NDS_10' => 'НДС 10',
			'NDS_18' => 'НДС 18',
			'NDS_10_CALCULATED' => 'НДС 10/110',
			'NDS_18_CALCULATED' => 'НДС 18/118'
		);
	}
    public function paymentIds() {
        $payments = WC_Payment_Gateways::instance();
        $paymentIds = $payments->get_payment_gateway_ids();//WC_Payment_Gateways::get_available_payment_gateways();
        
        foreach ($paymentIds as $key => $code) {
            $_paymentIds[$code] = $payments->payment_gateways[$key]->title;
        }
        return $_paymentIds;
	}

    public function fiscalize($order_id)
    {
        $order = wc_get_order($order_id);

        if (!$order) {
            return;
        }

        $status = $order->get_status();
        $payment = $order->get_payment_method();
        $payments_ids = get_option('dreamkas_payments_ids');

        if(!in_array($payment, explode(',', $payments_ids))) {
            return;
        }
        
        $tax_mode = get_option('dreamkas_tax_mode');
        $tax_type = get_option('dreamkas_tax_type');

        $items = array();
        if (sizeof($order->get_items()) > 0 ) {
            foreach ($order->get_items('line_item') as $product) {
                $product_tax_type = get_post_meta( $product['product_id'], 'dk_tax_type', true );
                $items[] = array(
                    "name"=> $product->get_name(),
                    "type"=> "COUNTABLE",
                    "quantity"=> $product['quantity'],
                    "price"=> intval(($product['total']+$product['total_tax'])/$product['quantity']*100),
                    "priceSum"=> ($product['total']+$product['total_tax'])*100,
                    "tax"=>  empty($product_tax_type)?$tax_type:$product_tax_type,//"$tax_type",
                    "taxSum"=> 0//$product['total_tax']*100
                );
            }
        }
        // shipping
        foreach ($order->get_items('shipping') as $item) {
                $items[] = array(
                    "name"=> 'Доставка',//$item->get_name(),
                    "type"=> "COUNTABLE",
                    "quantity"=> 1,
                    "price"=> round(($item['total']+$item['total_tax'])*100),
                    "priceSum"=> round(($item['total']+$item['total_tax'])*100),
                    "tax"=> "$tax_type",
                    "taxSum"=> 0//round($item['total_tax']*100)
                );
        }
        //fn_write_die($items);
        //fnd($items);
        $request = array(
            "deviceId" => get_option('dreamkas_device_id'),
            "type" => "SALE",
            "timeout" => 180,
            "taxMode" => get_option('dreamkas_tax_mode'),
            "positions" => $items,
            "payments" => array(
                array(
                    "sum" => $order->get_total()*100,
                    "type" => "CASHLESS"
                )
            ),
            "attributes" => array(
              "email" => $order->get_billing_email(),
              "phone" => $order->get_billing_phone(),
            ),
            "total" => array(
              "priceSum" => $order->get_total()*100
            )
        );

       //fnd($request,$order->get_items('line_item'),  $order);
        $ch = curl_init();

        $access_token = get_option('dreamkas_access_token');

        curl_setopt($ch, CURLOPT_HTTPHEADER, array(
            "Content-Type: application/json",
            "Authorization: Bearer $access_token"
        ));

        curl_setopt($ch, CURLOPT_URL, "https://kabinet.dreamkas.ru/api/receipts");
        curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
        curl_setopt($ch, CURLOPT_HEADER, FALSE);
        curl_setopt($ch, CURLOPT_POST, TRUE);
        curl_setopt($ch, CURLOPT_POSTFIELDS, json_encode($request));
        $response = curl_exec($ch);
        curl_close($ch);
               
        if(!empty($response)) {
            $response = json_decode($response, true);
            //fnd($response);
            /*$response = json_decode('{
                "id": "5956889136fdd7733f19cfe6",
                "createdAt": "2017-06-20 12:01:47.990Z",
                "status": "PENDING"
              }', true);*/

            if((substr($response['status'], 0, 1)==4)) {
                //$this->log->write('Dreamkas debug: ' . json_encode($response));
            } else {
                $dk_date = empty($response['createdAt'])?$response['completedAt']:$response['createdAt'];
                global $wpdb;

                $table_name = $wpdb->prefix . 'dreamkas';

                $exist_order_id = $wpdb->get_row( $wpdb->prepare( "SELECT order_id FROM {$table_name} WHERE `order_id` = %d LIMIT 1;", $order_id ) );

                if(empty($exist_order_id)) {
                    //$wpdb->query( "INSERT INTO $table_name ( `order_id` , `dk_id` , `dk_date`, `dk_status` ) VALUES ( '{$wpdb->blogid}', '{$wp_db_version}', NOW());" );
                    $wpdb->query("INSERT INTO `" . $wpdb->prefix . "dreamkas` SET `order_id` = '" . (int)$order_id . "', `dk_id` = '".$response['id']."', `dk_date` ='".$dk_date."', `dk_status` = '" . $response['status']. "'");
                } else {
                    $wpdb->query("UPDATE `" . $wpdb->prefix . "dreamkas` SET `order_id` = '" . (int)$order_id . "', `dk_id` = '".$response['id']."', `dk_date` ='".$dk_date."', `dk_status` = '" .$response['status']. "' WHERE order_id = '" . (int)$order_id. "'");
                }
            }
        }
    }

    public function add_query_vars($vars) {
        $vars[] = 'dreamkas';
        return $vars;
    }

    public static function add_endpoint() {
		add_rewrite_endpoint('dreamkas', EP_ALL);
    }

    public function handle_requests() {
        global $wp;

        if (empty($wp->query_vars['dreamkas'])) {
             return;
        }

        $dreamkas_action = strtolower(wc_clean( $wp->query_vars['dreamkas']));
        do_action('dreamkas_action_' . $dreamkas_action);
        die(-1);
    }

    public function action_success()
    {
        $this->handle_action('success');
    }

    public function action_fail()
    {
        //$this->handle_action('fail');
    }

    public function handle_action($action) {
        //do_action('dreamkas_report_update', intval($data['external_id']), $data['state'], $data);
    }
/*
    public function report_create($order_id, $request_check_data, $response_data, $error="")
    {
        $this->report->create($order_id, $request_check_data, $response_data, $error);
    }
*/
    /*
    public function report_update($order_id, $state, $report_data)
    {
        $this->report->update($order_id, $state, $report_data);
    }
     * 
     */
}

function Dreamkas() {
	return Dreamkas::instance();
}

$GLOBALS['dreamkas'] = Dreamkas();

add_action( 'woocommerce_admin_order_data_after_shipping_address', 'dreamkas_checkout_field_display_admin_order_meta', 10, 1 );

function dreamkas_checkout_field_display_admin_order_meta($order){

        global $wpdb;
        
        $table_name = $wpdb->prefix . 'dreamkas';
    
        $order_id = version_compare( WC_VERSION, '3.0.0', '<' ) ? $order->id : $order->get_id();

        $dk_data = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM {$table_name} WHERE `order_id` = %d LIMIT 1;", $order_id ) );
        
        if(!empty($dk_data) && $dk_data->dk_status != 'SUCCESS') {
            
            $dk_id = $dk_data->dk_id;
            
            $ch = curl_init();
            $access_token = get_option('dreamkas_access_token');

            curl_setopt($ch, CURLOPT_HTTPHEADER, array(
                "Content-Type: application/json",
                "Authorization: Bearer $access_token"
            ));

            curl_setopt($ch, CURLOPT_URL, "https://kabinet.dreamkas.ru/api/operations/".$dk_id);
            curl_setopt($ch, CURLOPT_RETURNTRANSFER, TRUE);
            curl_setopt($ch, CURLOPT_HEADER, FALSE);
            $response = curl_exec($ch);
            curl_close($ch);

            $response = json_decode($response, true);

            if((substr($response['status'], 0, 1)==4)) {
                //$this->log->write('Dreamkas debug: ' . json_encode($response));
            } else {
                $dk_date = empty($response['createdAt'])?$response['completedAt']:$response['createdAt'];
                $wpdb->query("UPDATE `" . $wpdb->prefix . "dreamkas` SET `order_id` = '" . (int)$order_id . "', `dk_id` = '".$response['id']."', `dk_date` ='".$dk_date."', `dk_status` = '" .$response['status']. "' WHERE order_id = '" . (int)$order_id. "'");
            }
            
        }
        
        $dk_data = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM {$table_name} WHERE `order_id` = %d LIMIT 1;", $order_id ) );
        
        if(!empty($dk_data->dk_status)) {
            echo '<p><strong>Дримкас статус:</strong> ' . $dk_data->dk_status . '</p>';
        }

    }
    
//add_filter( 'manage_book_posts_columns', 'set_custom_edit_book_columns' );
//add_action( 'manage_book_posts_custom_column' , 'custom_book_column', 10, 2 );

add_action( 'manage_shop_order_posts_custom_column', 'custom_status_column', 10, 2);
add_filter( 'manage_shop_order_posts_columns', 'set_custom_status_columns', 10, 2 );

function set_custom_status_columns($columns) {
    $columns['dk_status'] = 'Статус фискализации';
    //return $columns;
}

function custom_status_column( $column ) {
    
    global $post, $the_order;

    if ( empty( $the_order ) || $the_order->get_id() !== $post->ID ) {
            $the_order = wc_get_order( $post->ID );
    }
    
    $post_id =  $post->ID;
    
    switch ( $column ) {

        //case 'dk_status' :
        case 'order_title' :
            global $wpdb;

            $table_name = $wpdb->prefix . 'dreamkas';
            $dk_data = $wpdb->get_row( $wpdb->prepare( "SELECT * FROM {$table_name} WHERE `order_id` = %d LIMIT 1;", $post_id ) );
            if(!empty($dk_data->dk_status)) {
                echo '<p><strong>Статус фискализации:</strong> ' . $dk_data->dk_status . '</p>';
            }
            break;

    }
}

add_action( 'woocommerce_product_options_pricing', 'wc_dk_tax_type_product_field' );
function wc_dk_tax_type_product_field() {

    woocommerce_wp_select( array(
            'id'      => 'dk_tax_type',
            'label' => 'НДС товара при фискализации',
            'options' => Dreamkas()->taxTypes(),
    ) );

}
add_action( 'save_post', 'wc_dk_tax_type_save_product' );
function wc_dk_tax_type_save_product( $product_id ) {
    if ( defined( 'DOING_AUTOSAVE' ) && DOING_AUTOSAVE )
    return;
   
    if ( isset( $_POST['dk_tax_type'] ) ) {
        update_post_meta( $product_id, 'dk_tax_type', $_POST['dk_tax_type'] );
    } else {
        delete_post_meta( $product_id, 'dk_tax_type' );
    }
}